//  GFAX - Gnome fax application
//  Copyright (C) 2003 George A. Farris
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

namespace gfax {
	using System;
	using System.IO;
	using GLib;
	using Gtk;
	using Gnome;
	using Glade;
	using GConf.PropertyEditors;
	using GtkSharp;
	using System.Runtime.InteropServices;
	using System.Collections;
	using System.Reflection;

	
	//************************************************************************
	// Gfax class
	//
	// This is the main application window
	
	public class Gfax : Program
	{
		const string VERSION = "0.6.4";
		const string NAME = "Gfax";
		const string APPNAME = "Gfax";
		
		// Notebook pages
		const int MAIN_PAGE = 0;
		const int NEWFAX_PAGE = 1;
		const int PHONEBOOK_PAGE = 2;
		const int NEWPHONEBOOK_PAGE = 3;
		const int NEWNUMBER_PAGE = 4;
		
		const int COLUMN_0 = 0;
		const int COLUMN_1 = 1;
		const int COLUMN_2 = 2;
		const int COLUMN_3 = 3;		
		const int ALL_COLUMNS = -1;	
	
		// Strings to hold old contact info temporarily
		string tempName;
		string tempNumber;
		string tempCompany;
		
		bool restart = true;
		bool eventsEnabled = true;
		
		enum ActiveQ { done, send, receive };
		ActiveQ activeQ;
		
		Glade.XML gxml;
		
		[Glade.Widget] Gnome.AppBar Appbar;
						
		[Glade.Widget] Gtk.TreeView StatusList;
		[Glade.Widget] Gtk.Frame StatusFrame;
		[Glade.Widget] Gtk.Notebook MainNotebook;
		[Glade.Widget] Gtk.Button DeleteJobButton;
		[Glade.Widget] Gtk.TextView StatusText;
		[Glade.Widget] Gtk.TreeView PhoneBookList;
		[Glade.Widget] Gtk.TreeView PhoneBookItemList;
		[Glade.Widget] Gtk.TreeView NewPhbPhoneBookList;
				
		// Menu items
		[Glade.Widget] Gtk.CheckMenuItem AutoQRefreshCheckMenuItem;
		[Glade.Widget] Gtk.CheckMenuItem EmailNotificationCheckMenuItem;
		[Glade.Widget] Gtk.CheckMenuItem HiResolutionModeCheckMenuItem;
		[Glade.Widget] Gtk.CheckMenuItem LogEnabledCheckMenuItem;
		
		// Send Fax page widgets
		[Glade.Widget] Gtk.SpinButton DaySpinbutton;
		[Glade.Widget] Gtk.SpinButton HourSpinbutton;
		[Glade.Widget] Gtk.SpinButton MinuteSpinbutton;
		[Glade.Widget] Gtk.RadioButton SendNowRadiobutton;
		[Glade.Widget] Gtk.Table SendTable;
		//[Glade.Widget] Gnome.DateEdit SendDateEdit;
		[Glade.Widget] Gtk.Button SendSendfaxButton;
		[Glade.Widget] Gtk.Button SendClearButton;
		[Glade.Widget] Gtk.TreeView SendNumberList;
		[Glade.Widget] Gtk.Entry SendNumberEntry;
		[Glade.Widget] Gtk.Entry SendPSfileEntry;
		[Glade.Widget] Gtk.Button SendNumberAddButton;
		Gtk.ListStore sendstore;
		G_ListView sendview;	
		
		// Phone book page
		[Glade.Widget] Gtk.Button PhbDeletebookButton;
		[Glade.Widget] Gtk.Button PhbEditbookButton;
		
		// New phone book widgets
		[Glade.Widget] Gtk.Entry NewPhbNameEntry;
		[Glade.Widget] Gtk.Entry NewPhbPathEntry;
		[Glade.Widget] Gtk.RadioButton NewPhbGfaxRadioButton;
		[Glade.Widget] Gtk.RadioButton NewPhbGcardRadioButton;
		
		// Edit phone book entries widgets
		[Glade.Widget] Gtk.TreeView EditPhbList;
		[Glade.Widget] Gtk.Button EditPhbSaveButton;
		[Glade.Widget] Gtk.Button EditPhbDeleteButton;		
		[Glade.Widget] Gtk.Entry EditPhbNumberEntry;
		[Glade.Widget] Gtk.Entry EditPhbNameEntry;
		[Glade.Widget] Gtk.Entry EditPhbCompanyEntry;
		[Glade.Widget] Gtk.Entry EditPhbCurrentBookEntry;
		
		Gtk.Window MainWindow;
		Gtk.TreeModel BookModel;
		Gtk.ListStore BookStore, ListStore;	//phone: book and item store
		G_ListView bs, bsi, ls, lsi;
				
		Gtk.ListStore StatusStore;
		Gtk.TreeModel StatusModel;
		G_ListView lv;
		
		Phonebook[] myPhoneBooks;
		
		GConf.PropertyEditors.EditorShell shell;
		
						
		public Gfax (string fname, string[] args)
			: base (APPNAME, VERSION, Modules.UI, args, new object [0])
		{
			Phonebook[] pb;
			
			Application.Init ();
			
			if (!Directory.Exists(gfax.SPOOLDIR)) {
				G_Message gm = new G_Message("Your spool directory is missing! \n\n"
						+ "Please login as the \"root\" user and \ncreate the " 
						+ gfax.SPOOLDIR + " directory.\n\nAll users should be able to write to it.\n");
				return;
			}

			gxml = new Glade.XML (null, "gfax.glade", "Gfax", null);
			gxml.Autoconnect (this);

			// Set initial gui state as per preferences
			// GConf.PropertyEditors.EditorShell doesn't handle 
			// checkmenuitems;
			eventsEnabled = false;
		 		AutoQRefreshCheckMenuItem.Active = Settings.RefreshQueueEnabled;
				EmailNotificationCheckMenuItem.Active = Settings.EmailNotify;
				HiResolutionModeCheckMenuItem.Active = Settings.HiResolution;
				LogEnabledCheckMenuItem.Active = Settings.LogEnabled;
			eventsEnabled = true;
			DeleteJobButton.Sensitive = false;
			MainNotebook.ShowTabs = false;
			
			
			// Set the program icon
			Gdk.Pixbuf Icon = new Gdk.Pixbuf(null, "gfax.png");
			((Gtk.Window) gxml["Gfax"]).Icon = Icon;
			
			if (Settings.TransmitAgent == "hylafax") {
				StatusStore = new ListStore(
							typeof (string),
							typeof (string),
							typeof (string),
							typeof (string),
							typeof (string),
							typeof (string),
							typeof (string),
							typeof (string));
			
				lv = new G_ListView(StatusList, StatusStore);
				lv.AddColumnTitle("Jobid", 0, 0);
				lv.AddColumnTitle("Number",0, 1);
				lv.AddColumnTitle("Status",0, 2);
				lv.AddColumnTitle("Owner",0, 3);
				lv.AddColumnTitle("Pages",0, 4);
				lv.AddColumnTitle("Dials",0, 5);
				lv.AddColumnTitle("Send At",0, 6);
				lv.AddColumnTitle("Error",0, 7);

				StatusList.Selection.Changed += 
					new EventHandler(on_StatusList_selection);

				StatusList.HeadersVisible = true;
				StatusFrame.Label = "Send Queue";
				activeQ = ActiveQ.send;
				StatusList.Selection.Mode = SelectionMode.Multiple;				
				update_queue_status("sendq");
			}

			// setup the send fax list
			sendstore = new ListStore(
					typeof (Boolean),
					typeof (string),
					typeof (string),
					typeof (string));
			
			sendview = new G_ListView(SendNumberList, sendstore);
			sendview.AddColumnTitleToggle("Send", 0, COLUMN_0);
			sendview.AddColumnTitle("Phone Number", 0, COLUMN_1);
			sendview.AddColumnTitle("Organization", 0, COLUMN_2);
			sendview.AddColumnTitle("Contact", 0, COLUMN_3);
			SendNumberList.HeadersVisible = true;
			SendNumberList.Selection.Mode = SelectionMode.Multiple;
			
			
			// setup the phone book lists
			BookStore = new ListStore(typeof (string));
			
			PhoneBookList.HeadersVisible = false;
			PhoneBookList.Selection.Mode = SelectionMode.Multiple;
			PhoneBookList.Selection.Changed += 
					new EventHandler(on_PhoneBookList_selection);
			
			bs = new G_ListView(PhoneBookList, BookStore);
			// Got have a column title or things won't show up
			bs.AddColumnTitle("Phone books", 0, COLUMN_0);
			
			NewPhbPhoneBookList.HeadersVisible = false;
			NewPhbPhoneBookList.Selection.Mode = SelectionMode.Multiple;
			NewPhbPhoneBookList.Selection.Changed += 
					new EventHandler(on_NewPhbPhoneBookList_selection);
			
			bsi = new G_ListView(NewPhbPhoneBookList, BookStore);
			bsi.AddColumnTitle("Phone books", 0, COLUMN_0);
			
			ListStore = new ListStore(
					typeof (string),
					typeof (string),
					typeof (string));
			
			ls = new G_ListView(PhoneBookItemList, ListStore);
			ls.AddColumnTitle("Phone Number", 0, COLUMN_0);
			ls.AddColumnTitle("Organization", 0, COLUMN_1);
			ls.AddColumnTitle("Contact", 0, COLUMN_2);
			PhoneBookItemList.HeadersVisible = true;
			PhoneBookItemList.Selection.Mode = SelectionMode.Multiple;
			
			// phone book items on the edit entries page
			lsi = new G_ListView(EditPhbList, ListStore);
			lsi.AddColumnTitle("Phone Number", 0, COLUMN_0);
			lsi.AddColumnTitle("Organization", 0, COLUMN_1);
			lsi.AddColumnTitle("Contact", 0, COLUMN_2);
			EditPhbList.HeadersVisible = true;
			EditPhbList.Selection.Mode = SelectionMode.Multiple;
			EditPhbList.Selection.Changed += 
					new EventHandler(on_EditPhbList_selection);
			
			if (Settings.RefreshQueueEnabled)
				Gtk.Timeout.Add((uint)(Settings.RefreshQueueInterval * 1000), 
					new Gtk.Function(queue_refresh));
			
			update_status(Fax.get_server_status());
			
			Application.Run ();
		}

		//************************************************************************
		// queue_refresh
		//		returns true to continue timeout functioning
		//	
		// Queue_refresh connects to the hylafax server and retrieves
		// updated queue information on regular intervals.  This is called
		// from a GLib.TimeoutHandler
		public bool queue_refresh ()
		{
			if (!Settings.RefreshQueueEnabled)
				return false;
			
			// always do main status			
			update_status(Fax.get_server_status());	
			// now do queue
			switch (activeQ) {
				case ActiveQ.done :
					//Console.WriteLine("{0} queue is active", activeQ);
					update_queue_status("doneq");
					break;
				case ActiveQ.send :
					//Console.WriteLine("{0} queue is active", activeQ);
					update_queue_status("sendq");
					break;
				case ActiveQ.receive :
					//Console.WriteLine("{0} queue is active", activeQ);
					update_queue_status("recvq");
					break;
			}
			
			return true;	// must return true from timeout to keep it active
		}
		
		// Set to true if we need to restart for changed transport agent
		public bool Restart
		{
			get { return restart; }
			set { restart = value; }
		}
		
		

//========================= PRIVATE METHODS ====================================
		private void on_StatusList_selection (object o, EventArgs args)
		{
			DeleteJobButton.Sensitive = true;
		}
		
		private void on_exitButton_clicked (object o, EventArgs args)
		{
			Console.WriteLine("Goodbye...");
			Application.Quit();
		}

		private void on_mainWindow_delete_event (object o, DeleteEventArgs args) 
		{
			Application.Quit();
			args.RetVal = true;
		}

		private void on_exit1_activate (object o, EventArgs args)
		{
			Application.Quit();
			restart = true;
		}


//Main toolbar buttons
		private void on_MainStatusButton_clicked (object o, EventArgs args)
		{
			MainNotebook.CurrentPage = MAIN_PAGE;
		}

		private void on_NewFaxButton_clicked (object o, EventArgs args)
		{
			// Delete this stuff when Gnome.DateEdit begins to work.
			DateTime now = DateTime.Now;
			DaySpinbutton.Value = now.Day;
			HourSpinbutton.Value = now.Hour;
			MinuteSpinbutton.Value = now.Minute;
			SendNowRadiobutton.Active = true;
			SendTable.Sensitive = false;

			MainNotebook.CurrentPage = NEWFAX_PAGE;
			SendSendfaxButton.Sensitive = false;
			SendClearButton.Sensitive = false;
			gfax.Destinations.Clear();
			sendstore.Clear();
		}
		
		private void on_PreferencesButton_clicked (object o, EventArgs args)
		{
			GfaxPrefs gp = new GfaxPrefs();
		}

		private void on_PhonebookButton_clicked (object o, EventArgs args)
		{
			MainNotebook.CurrentPage = PHONEBOOK_PAGE;
			// unselect everything first
			PhoneBookList.Selection.UnselectAll();
			PhbDeletebookButton.Sensitive = false;
			PhbEditbookButton.Sensitive = false;			
			
			// populate the list
			ListStore.Clear();
			BookStore.Clear();
			myPhoneBooks = Phonetools.get_phonebooks();
			foreach (Phonebook p in myPhoneBooks)
				bs.AddTextToRow(p.Name);
		}
		
		
// Menu items selected =============================================
		private void on_newfax_activate (object o, EventArgs args)
		{
			MainNotebook.CurrentPage = NEWFAX_PAGE;
		}

		private void on_jobs1_activate (object o, EventArgs args)
		{
			//MainNotebook.CurrentPage = NEWFAX_PAGE;
		}

		private void on_auto_queue_refresh_check_menu_item_activate (object o, EventArgs args)
		{
			if (eventsEnabled) {
				Settings.RefreshQueueEnabled = !Settings.RefreshQueueEnabled;
				if (Settings.RefreshQueueEnabled)
					Gtk.Timeout.Add((uint)(Settings.RefreshQueueInterval * 1000), 
						new Gtk.Function(queue_refresh));
			}
		}

		private void on_log_enabled_check_menu_item_activate (object o, EventArgs args)
		{
			if (eventsEnabled)
				Settings.LogEnabled = !Settings.LogEnabled;
		}
		
		private void on_email_notification_check_menu_item_activate (object o, EventArgs args)
		{
			if (eventsEnabled)
				Settings.EmailNotify = !Settings.EmailNotify;
		}

		private void on_hi_resolution_mode_check_menu_item_activate (object o, EventArgs args)
		{
			if (eventsEnabled)
				Settings.HiResolution = !Settings.HiResolution;
		}

		private void on_preferences_activate (object o, EventArgs args)
		{
			GfaxPrefs gp = new GfaxPrefs();
		}
		
//		private void on_ssendNumberEntry_activate  (object o, EventArgs args)
//		{
//		}
		
		private void on_about_activate (object o, EventArgs args)
		{
			Gdk.Pixbuf icon = new Gdk.Pixbuf (null, "gfax.png");
			string[] authors = new string[] { "George Farris <george@gmsys.com>" };
			string[] documentors = new string[] {};
			string copyright = "Copyright (C) 2003 George Farris <george@gmsys.com>";
			string description = "A Facsimile application for GNOME";
			
			G_About a = new G_About (NAME, VERSION, copyright,description, authors,
					documentors, icon);
		}

//-------------------  MAIN PAGE CONTROL BAR -----------------------------------
		private void on_SendQButton_clicked (object o, EventArgs args)
		{
			StatusFrame.Label = "Send Queue";
			activeQ = ActiveQ.send;
			if (update_queue_status("sendq") > 0)
				DeleteJobButton.Sensitive = true;
			else {
				DeleteJobButton.Sensitive = false;
				
			}
		}
		
		private void on_DoneQButton_clicked (object o, EventArgs args)
		{
			StatusFrame.Label = "Done Queue";
			DeleteJobButton.Sensitive = false;
			activeQ = ActiveQ.done;
			update_queue_status("doneq");
		}
		
		private void on_ReceiveQButton_clicked (object o, EventArgs args)
		{
			StatusFrame.Label = "Receive Queue";
			DeleteJobButton.Sensitive = false;
			activeQ = ActiveQ.receive;
		}

		private void on_DeleteJobButton_clicked (object o, EventArgs args)
		{
			// get the selected jobs
			ArrayList al;
			
			// get a list of jobids to kill
			al = lv.GetSelections(COLUMN_0);
			
			IEnumerator enu = al.GetEnumerator();
			while (	enu.MoveNext() ) {
				Fax.delete_job((string)enu.Current);
			}
			
			switch (activeQ) {
					case ActiveQ.done :
						update_queue_status("doneq");
						break;
					case ActiveQ.send :
						update_queue_status("sendq");
						break;
					case ActiveQ.receive :
						update_queue_status("recvq");
						break;
			}
			
			DeleteJobButton.Sensitive = false;
		}

		
		
//-----------------------------  SEND PAGE -------------------------------------
		private void on_SendSendfaxButton_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter iter = new Gtk.TreeIter();
			ArrayList rows = new ArrayList();
			string filename;
			
			filename = SendPSfileEntry.Text;
			if (filename == "") {
				G_Message gm = new G_Message("Filename is empty!");
				return;
			}
			// (TODO better checking) If not a postscript file.
			if (Path.GetExtension(filename) != ".ps") {
				G_Message gm = new G_Message("File is not in Postscript format!");
				return;
			}
			
			// clear all the distinations, it's a little wierd yup
			gfax.Destinations.Clear();
			
			// Get the first row.
			sendstore.GetIterFirst(out iter);

			try {
				
				if ( (bool)sendstore.GetValue(iter, 0) ) {		// if send is true (toggle set)
										
					Contact c = new Contact();
					c.PhoneNumber = (string)sendstore.GetValue(iter, 1);	// number
					c.Organization = (string)sendstore.GetValue(iter, 2);	// organization
					c.ContactPerson = (string)sendstore.GetValue(iter, 3);	// contact
										
					rows.Add(c);
				}
			}
			catch (Exception e) {}
			
			// get the rest of the rows			
			while (sendstore.IterNext(ref iter)) {
				try {
					if ( (bool)sendstore.GetValue(iter, 0) ) {		// if send is true (toggle set)
										
						Contact c = new Contact();
						c.PhoneNumber = (string)sendstore.GetValue(iter, 1);	// number
						c.Organization = (string)sendstore.GetValue(iter, 2);	// organization
						c.ContactPerson = (string)sendstore.GetValue(iter, 3);	// contact
										
						rows.Add(c);
					}
				}
				catch (Exception e) {}
			}
			
			
			if (!SendNowRadiobutton.Active) {
				// TODO check globalization
				DateTime now = DateTime.Now;
				Console.WriteLine("[SendFax] Date is {0}", now);
				
				int nextMonth = 0;
				if (DaySpinbutton.Value < now.Day)
					nextMonth = 1;

				DateTime temp = new DateTime(
						now.Year,
						now.Month + nextMonth,
						(int)DaySpinbutton.Value,
						(int)HourSpinbutton.Value,
						(int)MinuteSpinbutton.Value,
						0,0);
								
				// Convert to UTC for Hylafax
				gfax.timeToSend = temp.ToUniversalTime();				
				//Console.WriteLine("[SendFax] GMT Date is {0}", gfax.timeToSend);
			}
			
			
			gfax.Destinations = rows;
			// get the time to send
			
			if (gfax.Destinations.Count > 0)
				Fax.sendfax(filename);
			StatusFrame.Label = "Send Queue";
			DeleteJobButton.Sensitive = true;
			activeQ = ActiveQ.send;
			update_queue_status("sendq");
			MainNotebook.CurrentPage = MAIN_PAGE;
		}
		
		private void on_SendClearButton_clicked (object o, EventArgs args)
		{
			sendstore.Clear();
			SendSendfaxButton.Sensitive = false;
			SendClearButton.Sensitive = false;
			gfax.Destinations.Clear();	
		}
		private void on_SendNumberAddButton_clicked (object o, EventArgs args)
		{
			if (SendNumberEntry.Text != "") {
				SendSendfaxButton.Sensitive = true;
				SendClearButton.Sensitive = true;
				sendview.AddTextToRow(SendNumberEntry.Text);
				SendNumberEntry.Text = "";
			} else {
				GfaxSendPhoneBook gfs = new GfaxSendPhoneBook(gxml, "gfax");
				
				if (gfax.Destinations.Count > 0) {
					Gtk.TreeIter iter = new Gtk.TreeIter();
                    sendstore.Clear();
                    IEnumerator enu = gfax.Destinations.GetEnumerator();
                    while (enu.MoveNext()) {
                            iter = sendstore.AppendValues(true,
                                    ((Contact)enu.Current).PhoneNumber,
                                    ((Contact)enu.Current).Organization,
                                    ((Contact)enu.Current).ContactPerson );
                            SendNumberList.Model = sendstore;
					}
					SendSendfaxButton.Sensitive = true;
					SendClearButton.Sensitive = true;
				}
			}
		}

		private void on_SendNumberList_button_press_event (object o, EventArgs args)
		{
		}
		
		private void on_SendNumberEntry_activate (object o, EventArgs args)
		{
			if (SendNumberEntry.Text != "") {
				SendSendfaxButton.Sensitive = true;
				SendClearButton.Sensitive = true;

				Gtk.TreeIter iter = new Gtk.TreeIter();

				iter = sendstore.AppendValues(true,
						SendNumberEntry.Text,
						"",
						"");
				
				
				SendNumberList.Model = sendstore;
								
				//sendview.AddTextToRow(SendNumberEntry.Text);
				Contact c = new Contact();
				c.PhoneNumber = SendNumberEntry.Text;
				c.Organization = "";
				c.ContactPerson = "";
				gfax.Destinations.Add(c);

				SendNumberEntry.Text = "";
			}
		}
		
		// Enable Date time setting
		private void on_SendNowRadiobutton_toggled (object o, EventArgs args)
		{
			SendTable.Sensitive = !SendTable.Sensitive;
		}


//---------------------------PHONE BOOK PAGE -----------------------------------
		private void on_PhbNewbookButton_clicked (object o, EventArgs args)
		{
			// Unselect any phone book otherwise we get gtk errors.
			PhoneBookList.Selection.UnselectAll();
			PhbDeletebookButton.Sensitive = false;
			PhbEditbookButton.Sensitive = false;			

			// run the wizard
			NewPhoneBook nphb = new NewPhoneBook ();
            nphb.Run();

			if ( nphb.PhoneBookName != null )	{	// don't do this if cancelled
				Phonebook ph = new Phonebook();
			
				// TODO  get location from gconf
				string HOMEDIR = Environment.GetEnvironmentVariable("HOME");			
				
				ph.Path = HOMEDIR + "/.etc/gfax/" + nphb.PhoneBookName;
				ph.Name = nphb.PhoneBookName;	// get the new book name
				ph.Type = nphb.PhoneBookType;
				Phonetools.add_book(ph);
			
				myPhoneBooks = Phonetools.get_phonebooks();
				ListStore.Clear();
				BookStore.Clear();
			
				foreach (Phonebook p in myPhoneBooks) {
					bsi.AddTextToRow(p.Name);
				}
			
				MainNotebook.CurrentPage = MAIN_PAGE;
				MainNotebook.CurrentPage = PHONEBOOK_PAGE;
			}
        }
		
		
		private void on_PhbDeletebookButton_clicked (object o, EventArgs args)
		{
			string book = bs.GetSingleSelection();
			if (book == null)
				return;
			Phonetools.delete_book(book); 
			bs.RemoveSelectedRow();
			// now reload
			myPhoneBooks = Phonetools.get_phonebooks();
			ListStore.Clear();
		}
		
		private void on_PhbEditbookButton_clicked (object o, EventArgs args)
		{
			string book;
			
			book = bs.GetSingleSelection();
			if (book == null)
				return;
				
			MainNotebook.CurrentPage = NEWNUMBER_PAGE;
			EditPhbCurrentBookEntry.Text = book;
			EditPhbNumberEntry.HasFocus = true;
			EditPhbSaveButton.Sensitive = false;
			EditPhbDeleteButton.Sensitive = false;
			
			// load the list
			foreach (Phonebook p in myPhoneBooks)
				if (p.Name == book)
					load_phone_book(p, lsi);

		}
		
		// load the selected phone book
		// Since we have SelectionMode.Multiple turned on we have to 
		// jump through these hoops in GTK-2.0 to get a selection
		private void on_PhoneBookList_selection(object o, EventArgs args)
		{	
			PhbDeletebookButton.Sensitive = true;
			PhbEditbookButton.Sensitive = true;			
			// Ok now we can finally load the phone book
			foreach (Phonebook p in myPhoneBooks)
				if (p.Name == bs.GetSingleSelection())
					load_phone_book(p, ls);
		}

		// loads the phone book into list_store
		private void load_phone_book(Phonebook p, G_ListView lv)
		{
			ArrayList contacts = null;
			string name, number, company;
			
			// Clear the list_store
			ListStore.Clear();
			
			contacts = Phonetools.get_contacts(p);
			
			if (contacts != null) {
				IEnumerator enu = contacts.GetEnumerator();
				while ( enu.MoveNext() ) {
					Contact c = new Contact();
					c = (Contact)enu.Current;
					lv.AddTextToRow(c.Organization, c.PhoneNumber, c.ContactPerson);
				}
			}
		}






//---------------------------NEW PHONE BOOK PAGE -------------------------------
// WE DON'T USE THIS ANYMORE

		private void on_NewPhbSaveButton_clicked (object o, EventArgs args)
		{
			Phonebook ph = new Phonebook();
			
			ph.Name = NewPhbNameEntry.Text;
			ph.Path = NewPhbPathEntry.Text;
			if (NewPhbGfaxRadioButton.Active)
				ph.Type = "gfax";
			
			Phonetools.add_book(ph);
			
			NewPhbNameEntry.Text = "";
			NewPhbPathEntry.Text = "";
			myPhoneBooks = Phonetools.get_phonebooks();
			BookStore.Clear();
			foreach (Phonebook p in myPhoneBooks) {
				bsi.AddTextToRow(p.Name);
			}
		}
		private void on_NewPhbClearButton_clicked (object o, EventArgs args)
		{
			NewPhbNameEntry.Text = "";
			NewPhbPathEntry.Text = "";
		}
		private void on_NewPhbDeleteButton_clicked (object o, EventArgs args)
		{
			string book = bsi.GetSingleSelection();
			if (book == null)
				return;
			Phonetools.delete_book(book); 
			bsi.RemoveSelectedRow();
			// now reload
			myPhoneBooks = Phonetools.get_phonebooks();

			NewPhbNameEntry.Text = "";
			NewPhbPathEntry.Text = "";

		}

		private void on_NewPhbPhoneBookList_selection (object o, EventArgs args)
		{
			foreach (Phonebook p in myPhoneBooks)
				if (p.Name == bsi.GetSingleSelection()) {
					NewPhbNameEntry.Text = p.Name;
					NewPhbPathEntry.Text = p.Path;
				}
			
		}
		
		
		
		
		
		
		
//---------------------------PHONE BOOK ITEMS PAGE ----------------------------
		private void on_EditPhbSaveButton_clicked (object o, EventArgs args)
		{
			ArrayList rows;
			ArrayList contacts = new ArrayList();
			string book;
			
			book = EditPhbCurrentBookEntry.Text;
			
			// just get all items and save
			EditPhbSaveButton.Sensitive = false;
			eventsEnabled = false;
			EditPhbCompanyEntry.Text = "";
			EditPhbNumberEntry.Text = "";
			EditPhbNameEntry.Text = "";
			
			rows = lsi.GetAllRows();
						
			IEnumerator enu = rows.GetEnumerator();
    	  	while ( enu.MoveNext() ) {
				Contact c = new Contact();
				c.Organization = ((string[])enu.Current)[0];
				c.PhoneNumber = ((string[])enu.Current)[1];
				c.ContactPerson = ((string[])enu.Current)[2];
				contacts.Add(c);
			}
			
			Phonetools.save_phonebook_items(book, contacts);
			EditPhbList.Selection.UnselectAll();
			EditPhbDeleteButton.Sensitive = false;
		}
		
		
		private void on_EditPhbDeleteButton_clicked (object o, EventArgs args)
		{
			lsi.RemoveSelectedRow();
			eventsEnabled = false;
			EditPhbCompanyEntry.Text = "";
			EditPhbNumberEntry.Text = "";
			EditPhbNameEntry.Text = "";
			EditPhbSaveButton.Sensitive = true;
			EditPhbDeleteButton.Sensitive = false;			
		}
		
		private void on_EditPhbDeleteAllButton_clicked (object o, EventArgs args)
		{
			ListStore.Clear();
			EditPhbCompanyEntry.Text = "";
			EditPhbNumberEntry.Text = "";
			EditPhbNameEntry.Text = "";
		}
		
		private void on_EditPhbClearButton_clicked (object o, EventArgs args)
		{
			eventsEnabled = false;
			EditPhbCompanyEntry.Text = "";
			EditPhbNumberEntry.Text = "";
			EditPhbNameEntry.Text = "";
		}
		
		private void on_EditPhbAddButton_clicked (object o, EventArgs args)
		{
			lsi.AddTextToRow(EditPhbCompanyEntry.Text,
					EditPhbNumberEntry.Text,
					EditPhbNameEntry.Text);
					
			eventsEnabled = false;  // no updates till selected
			EditPhbCompanyEntry.Text = "";
			EditPhbNumberEntry.Text = "";
			EditPhbNameEntry.Text = "";
			// ok new entry now
			EditPhbNumberEntry.HasFocus = true;
		}

		private void on_EditPhbNumberEntry_changed (object o, EventArgs args)
		{		
			if (eventsEnabled) {
				EditPhbSaveButton.Sensitive = true;
				lsi.UpdateColumnText(tempNumber, EditPhbNumberEntry.Text, COLUMN_1);
				// update this so we can keep changing if so desired
				tempNumber = EditPhbNumberEntry.Text;
			}
		}
		private void on_EditPhbNameEntry_changed (object o, EventArgs args)
		{
			if (eventsEnabled) {
				EditPhbSaveButton.Sensitive = true;
				lsi.UpdateColumnText(tempName, EditPhbNameEntry.Text, COLUMN_2);
				// update this so we can keep changing if so desired
				tempName = EditPhbNameEntry.Text;
			}
		}
		private void on_EditPhbCompanyEntry_changed (object o, EventArgs args)
		{
			if (eventsEnabled) {
				EditPhbSaveButton.Sensitive = true;
				lsi.UpdateColumnText(tempCompany, EditPhbCompanyEntry.Text, COLUMN_0);
				// update this so we can keep changing if so desired
				tempCompany = EditPhbCompanyEntry.Text;
			}
		}
		
		private void on_EditPhbList_selection(object o, EventArgs args)
		{	
			ArrayList al;
			
			eventsEnabled = false;  // no updates			
			al = lsi.GetSelections(ALL_COLUMNS);
			
			// if single contact selected
			if (al.Count == 3) {
				IEnumerator enu = al.GetEnumerator();
				enu.MoveNext();
				EditPhbCompanyEntry.Text = (string)enu.Current;
				tempCompany = (string)enu.Current;
				enu.MoveNext();
				EditPhbNumberEntry.Text = (string)enu.Current;
				tempNumber = (string)enu.Current;
				enu.MoveNext();
				EditPhbNameEntry.Text = (string)enu.Current;
				tempName = (string)enu.Current;
				EditPhbDeleteButton.Sensitive = true;
			}
			
			eventsEnabled = true;
		}
				
		
		
//--------------------------- SUPPORT FUNCTIONS -------------------------------
		
		
		// Updates the status text widget
		private void update_status (string s)
		{
			TextBuffer tb = new TextBuffer(new TextTagTable());
			tb.SetText(s);
			StatusText.Buffer = tb;
		}
		
		private int update_queue_status(string queue)
		{
			if (Settings.TransmitAgent == "hylafax") {
				StatusStore.Clear();
				HylafaxQueue q;
				
				// update status bar
				Appbar.ClearStack();
				Appbar.Push("Refreshing queue...");
				Appbar.Refresh();
				//GLib.MainContext.Iteration ();
				while (Gtk.Application.EventsPending ())
                	Gtk.Application.RunIteration ();
					
				ArrayList reply = Fax.get_queue_status(queue);
				
				if (reply.Count > 0) {
					IEnumerator enu = reply.GetEnumerator();
    	  			while ( enu.MoveNext() ) {
    	     			q = (HylafaxQueue)enu.Current;
						lv.AddTextToRow(
								q.Jobid,
								q.Number,
								q.Status,
								q.Owner,
								q.Pages,
								q.Dials,
								q.Sendat,
								q.Error);
					}
					
				}
				Appbar.ClearStack();
				if (q.Jobid != "") {
					Appbar.Push("There are " + reply.Count + " jobs in the queue");
					Appbar.Refresh();
					((Gtk.Window) gxml["Gfax"]).Title = "Gfax (" + reply.Count + ")";
					return reply.Count;
				} else {
					Appbar.Push("There are 0 jobs in the queue");
					Appbar.Refresh();
					((Gtk.Window) gxml["Gfax"]).Title = "Gfax";
				}
				
			}
			StatusStore.Clear();
			return 0;
		}
	}

	//************************************************************************
    // NewPhoneBook class
    //
    // A new phone book druid, should make it easier for people to create
	// new phone books.  The old way was a little confusing.
    //
    public class NewPhoneBook
    {
		[Glade.Widget] Gtk.Dialog NewPhoneBookDialog;
		[Glade.Widget] Gnome.Druid NewPhoneBookDruid;
        [Glade.Widget] Gtk.RadioButton GfaxRadioButton;
        [Glade.Widget] Gtk.RadioButton EvolutionRadioButton;
        [Glade.Widget] Gtk.RadioButton DatabaseRadioButton;
        [Glade.Widget] Gtk.RadioButton LDAPRadioButton;
        [Glade.Widget] Gtk.Entry NewPhoneBookNameEntry;
 
		// Properties
		string phonebookname;
		string phonebooktype;
		
        public NewPhoneBook ()
        {
				Glade.XML xml = new Glade.XML (null, "gfax.glade","NewPhoneBookDialog",null);
                xml.Autoconnect (this);

                GfaxRadioButton.Active = true;
                 
                // turn these off until somewhere near supported
                DatabaseRadioButton.Visible = false;
                LDAPRadioButton.Visible = false;
								
				NewPhoneBookDruid.ShowAll();
        }
        
		public string PhoneBookName
		{
			get { return phonebookname; }
		}
		
		public string PhoneBookType
		{
			get { return phonebooktype; }
		}
		
        public void Run()
        {
            NewPhoneBookDialog.Run();
        }
              
        private void on_NewPhoneBookDialog_delete_event (object o, DeleteEventArgs args)
		{
			NewPhoneBookDialog.Hide();
			NewPhoneBookDialog.Dispose();
            args.RetVal = true;
        }

		private void on_NewPhoneBookDruidEdge_finish (object o, EventArgs args)
        {
            phonebookname = NewPhoneBookNameEntry.Text;
             
            if (GfaxRadioButton.Active)
                    phonebooktype = "gfax";
            else if (EvolutionRadioButton.Active)
                    phonebooktype = "evolution";
            else if (DatabaseRadioButton.Active)
                    phonebooktype = "sql";
            else if (LDAPRadioButton.Active)
                    phonebooktype = "ldap";

			NewPhoneBookDialog.Hide();
			NewPhoneBookDialog.Dispose();
        }

        private void on_NewPhoneBookDruid_cancel (object o, EventArgs args)
        {
			NewPhoneBookDialog.Hide();
			NewPhoneBookDialog.Dispose();
		}

    }



	//************************************************************************
	// GfaxSend class
	//
	// This is a druid that trys to automate the sending of a facsimile to
	// make it easier on the novice computer user.
	
	public class GfaxSend // : Program
	{
		const string APPNAME = "Gfax";
		
		const int COLUMN_0 = 0;
		const int COLUMN_1 = 1;
		const int COLUMN_2 = 2;
		const int COLUMN_3 = 3;
		const int ALL_COLUMNS = -1;	
		
		
		[Glade.Widget] Gtk.Button PhoneButton;
		[Glade.Widget] Gtk.Entry NumberEntry;
		[Glade.Widget] Gtk.Entry EmailEntry;
		[Glade.Widget] Gtk.CheckButton ResolutionCheckbutton;
		[Glade.Widget] Gtk.CheckButton EmailCheckbutton;
		[Glade.Widget] Gtk.TreeView ItemTreeview;
		[Glade.Widget] Gtk.RadioButton SendRadiobutton;
		[Glade.Widget] Gtk.SpinButton DaySpinbutton;
		[Glade.Widget] Gtk.SpinButton HourSpinbutton;
		[Glade.Widget] Gtk.SpinButton MinuteSpinbutton;
		[Glade.Widget] Gtk.Table SendTable;

		Gtk.ListStore ItemStore;
		G_ListView ItemListview;
		
		GConf.PropertyEditors.EditorShell shell;		
		
		Glade.XML gxml;
		string filename;
		bool dosend;
		
		public GfaxSend (string fname, string[] args)
			//: base (APPNAME, "", Modules.UI, args, new object [0])
		{
			
			filename = fname;
		
			Application.Init ();
			gxml = new Glade.XML (null, "send-druid.glade","MainWindow",null);
			gxml.Autoconnect (this);
			
						
			// Set the program icon
			Gdk.Pixbuf Icon = new Gdk.Pixbuf(null, "gfax.png");
			((Gtk.Window) gxml["MainWindow"]).Icon = Icon;
			
			Gdk.Pixbuf Image = new Gdk.Pixbuf(null, "send.png");
			((Gnome.DruidPageEdge) gxml["druidpagestart"]).Logo = Icon;
			((Gnome.DruidPageEdge) gxml["druidpagestart"]).Watermark = Image;
			((Gnome.Druid) gxml["send_druid"]).HasFocus = true;
			
			ItemStore = new ListStore(
					typeof (Boolean),
					typeof (string),
					typeof (string),
					typeof (string));
			
			ItemListview = new G_ListView(ItemTreeview, ItemStore);
			
			ItemListview.AddColumnTitleToggle("Send", 0, COLUMN_0);
			ItemListview.AddColumnTitle("Phone Number", 0, COLUMN_1);
			ItemListview.AddColumnTitle("Organization", 0, COLUMN_2);
			ItemListview.AddColumnTitle("Contact", 0, COLUMN_3);
			
			ItemTreeview.HeadersVisible = true;
			//ItemTreeview.Selection.Mode = SelectionMode.Multiple;
			
			ResolutionCheckbutton.Active = Settings.HiResolution;
			EmailCheckbutton.Active = Settings.EmailNotify;
			EmailEntry.Text = Settings.EmailAddress;
			
			// Delete this stuff when Gnome.DateEdit begins to work.
			DateTime now = DateTime.Now;
			DaySpinbutton.Value = now.Day;
			HourSpinbutton.Value = now.Hour;
			MinuteSpinbutton.Value = now.Minute;
			SendRadiobutton.Active = true;
			SendTable.Sensitive = false;


			((Gnome.Druid) gxml["send_druid"]).ShowAll();
			((Gnome.Druid) gxml["send_druid"]).HasFocus = true;
			Application.Run ();
		}

		public bool DoSend
		{
			get { return dosend; }
		}
		
		private void on_window1_delete_event (object o, DeleteEventArgs args) 
		{
			dosend = false;		
			((Gtk.Window) gxml["MainWindow"]).Destroy();			
			Application.Quit ();
			args.RetVal = true;
		}
		
		private void on_druidpagestart_next(object o, EventArgs args)
		{
			NumberEntry.HasFocus = true;
		}
		
		private void on_send_druid_cancel (object o, EventArgs args) 
		{
			dosend = false;
			((Gtk.Window) gxml["MainWindow"]).Destroy();
			Application.Quit ();
		}
		
		private void on_NumberEntry_changed (object o, EventArgs args) 
		{
			if (NumberEntry.Text == "\t")
				((Gnome.Druid) gxml["send_druid"]).HasFocus = true;
		}

		private void on_NumberEntry_activate (object o, EventArgs args) 
		{
			Gtk.TreeIter iter = new Gtk.TreeIter();
			
			iter = ItemStore.AppendValues(true, NumberEntry.Text, "", "");
			ItemTreeview.Model = ItemStore;
			NumberEntry.Text = "";
		}
		
		private void on_SendRadiobutton_toggled (object o, EventArgs args)
		{
			SendTable.Sensitive = !SendTable.Sensitive;
		}

		private void on_phone_button_clicked (object o, EventArgs args) 
		{
			Gtk.TreeIter iter = new Gtk.TreeIter();
			// display the phone book, when we return gfax.Destinations 
			// will be set with contacts.
			GfaxSendPhoneBook gfs = new GfaxSendPhoneBook(gxml, "gfaxsend");
			
			ItemStore.Clear();

			if (gfax.Destinations.Count > 0) {
				IEnumerator enu = gfax.Destinations.GetEnumerator();
    	  		
				while ( enu.MoveNext() ) {
					iter = ItemStore.AppendValues(true,
							((Contact)enu.Current).PhoneNumber,
							((Contact)enu.Current).Organization,
							((Contact)enu.Current).ContactPerson);
				
					ItemTreeview.Model = ItemStore;
				}
			}
		}

		
		// When finished gfax.Destinations will contain a list of contacts
		// of Phonebook.Contact type.
		private void on_druidfinish_finish (object o, EventArgs args) 
		{	
			Gtk.TreeIter iter = new Gtk.TreeIter();
			ArrayList rows = new ArrayList();
			
			// clear all the distinations, it's a little wierd yup
			gfax.Destinations.Clear();
			
			// Get the first row.
			ItemStore.GetIterFirst(out iter);

			try {
				if ( (bool)ItemStore.GetValue(iter, 0) ) {		// if send is true (toggle set)
										
					Contact c = new Contact();
					c.PhoneNumber = (string)ItemStore.GetValue(iter, 1);	// number
					c.Organization = (string)ItemStore.GetValue(iter, 2);	// organization
					c.ContactPerson = (string)ItemStore.GetValue(iter, 3);	// contact
										
					rows.Add(c);
				}
			}
			catch (Exception e) {}
			
			// get the rest of the rows			
			while (ItemStore.IterNext(ref iter)) {

				try {
					if ( (bool)ItemStore.GetValue(iter, 0) ) {		// if send is true (toggle set)
										
						Contact c = new Contact();
						c.PhoneNumber = (string)ItemStore.GetValue(iter, 1);	// number
						c.Organization = (string)ItemStore.GetValue(iter, 2);	// organization
						c.ContactPerson = (string)ItemStore.GetValue(iter, 3);	// contact
										
						rows.Add(c);
					}
				}
				catch (Exception e) {}
			}


			if (!SendRadiobutton.Active) {
				// TODO check globalization
				DateTime now = DateTime.Now;
								
				int nextMonth = 0;
				if (DaySpinbutton.Value < now.Day)
					nextMonth = 1;

				DateTime temp = new DateTime(
						now.Year,
						now.Month + nextMonth,
						(int)DaySpinbutton.Value,
						(int)HourSpinbutton.Value,
						(int)MinuteSpinbutton.Value,
						0,0);
								
				// Convert to UTC for Hylafax
				gfax.timeToSend = temp.ToUniversalTime();				
			}

			gfax.Destinations = rows;
			
			//get the fine resolution status
			gfax.sendWizardResolution = ResolutionCheckbutton.Active;
			//get the email flag and email address
			gfax.sendWizardEmailNotify = EmailCheckbutton.Active;
			gfax.sendWizardEmailAddress = EmailEntry.Text;				
			
			if ( gfax.Destinations.Count > 0 ) 
				dosend = true;	// yes send the fax
				
			((Gtk.Window) gxml["MainWindow"]).Hide();
			Application.Quit();
		}
	}

	
	
	
	//************************************************************************
	// GfaxPrefs class
	//
	// This is the preferences window.
	public class GfaxPrefs
	{
		Glade.XML gxml;
		[Glade.Widget] Gtk.Dialog PrefsDialog;
		// System Tab
		[Glade.Widget] Gtk.RadioButton HylafaxRadioButton;
		[Glade.Widget] Gtk.RadioButton MgettyRadioButton;
		[Glade.Widget] Gtk.RadioButton EfaxRadioButton;
		[Glade.Widget] Gtk.Entry FaxNumberEntry;
		[Glade.Widget] Gtk.Entry DialPrefixEntry;

		[Glade.Widget] Gtk.Entry ModemEntry;
		//Hylafax Tab
		[Glade.Widget] Gtk.Entry HylafaxHostnameEntry;
		[Glade.Widget] Gtk.Entry HylafaxPortEntry;
		[Glade.Widget] Gtk.Entry HylafaxUsernameEntry;
		//System Tab
		[Glade.Widget] Gtk.CheckButton EmailNotifyCheckButton;
		[Glade.Widget] Gtk.Entry	EmailAddressEntry;
		[Glade.Widget] Gtk.CheckButton SendNowCheckButton;
		[Glade.Widget] Gtk.CheckButton FaxLogCheckButton;
		[Glade.Widget] Gtk.CheckButton CoverPageCheckButton;
		[Glade.Widget] Gtk.CheckButton HiResCheckButton;

		// we set this otherwise setting inital values will wipe
		// out all the other settings.
		bool eventsEnabled = false; 
		
		public GfaxPrefs ()
		{
			gxml = new Glade.XML (null, "gfax.glade","PrefsDialog",null);
			//GConf.PropertEditors
			EditorShell shell = new EditorShell (gxml);
			gxml.Autoconnect(this);
			
			
			// System Tab
			if (Settings.TransmitAgent == "hylafax")
				HylafaxRadioButton.Active = true;
			else if (Settings.TransmitAgent == "mgetty")
				MgettyRadioButton.Active = true;
			else if (Settings.TransmitAgent == "efax")
				EfaxRadioButton.Active = true;

			shell.Add(SettingKeys.FaxNumber, "FaxNumberEntry");
			shell.Add(SettingKeys.PhonePrefix, "DialPrefixEntry");
			
			// Hylafax Tab
			shell.Add(SettingKeys.Hostname, "HylafaxHostnameEntry");
			shell.Add(SettingKeys.Port, "HylafaxPortEntry");
			shell.Add(SettingKeys.Username, "HylafaxUsernameEntry");
			
			// Mgetty Tab
			
			// Efax Tab
			
			// User Tab

			shell.Add(SettingKeys.EmailNotify, "EmailNotifyCheckButton");
			shell.Add(SettingKeys.EmailAddress, "EmailAddressEntry");
			shell.Add(SettingKeys.SendNow, "SendNowCheckButton");
			shell.Add(SettingKeys.LogEnabled, "FaxLogCheckButton");
			shell.Add(SettingKeys.CoverPage, "CoverPageCheckButton");
			shell.Add(SettingKeys.HiResolution, "HiResCheckButton");

			eventsEnabled = true;

		}

		private void on_PrefsDialog_delete_event (object o, DeleteEventArgs args) 
		{
			PrefsDialog.Destroy();
			args.RetVal = true;
		}

		private void on_CloseButton_clicked (object o, EventArgs args) 
		{
			PrefsDialog.Destroy();
		}
		private void on_HelpButton_clicked (object o, EventArgs args) 
		{
			
		}
		
		private void system_tab_changed (object o, EventArgs args) 
		{
			if (eventsEnabled) {
				if (HylafaxRadioButton.Active)
					Settings.TransmitAgent = "hylafax";
				else if (MgettyRadioButton.Active)
					Settings.TransmitAgent = "mgetty";
				else if (EfaxRadioButton.Active)
					Settings.TransmitAgent = "efax";

				Settings.FaxNumber = FaxNumberEntry.Text;
				Settings.PhonePrefix = DialPrefixEntry.Text;
			}
		}
		
		private void hylafax_tab_changed (object o, EventArgs args) 
		{
			
		}
		private void mgetty_tab_changed (object o, EventArgs args) 
		{
			
		}
		private void efax_tab_changed (object o, EventArgs args) 
		{
		}
		private void user_tab_changed (object o, EventArgs args) 
		{
			if (eventsEnabled) {
				Settings.EmailNotify = EmailNotifyCheckButton.Active;
				Settings.EmailAddress = EmailAddressEntry.Text;
				Settings.SendNow = SendNowCheckButton.Active;
				Settings.LogEnabled = FaxLogCheckButton.Active;
				Settings.CoverPage = CoverPageCheckButton.Active;
				Settings.HiResolution = HiResCheckButton.Active;
			}
		}

		private void mgetty_setup_changed (object o, EventArgs args) 
		{
		}
		private void efax_setup_changed (object o, EventArgs args) 
		{
		}
		private void setup_changed (object o, EventArgs args) 
		{
		}

	}



	//************************************************************************
	// GfaxFirstPrefs class
	//
	// This is the first run preferences window.
	public class GfaxFirstPrefs
	{
		Glade.XML gxml;
		[Glade.Widget] Gtk.Window FirstTimeWindow;
		[Glade.Widget] Gnome.Druid FirstTimeDruid;
		[Glade.Widget] Gtk.RadioButton HylafaxRadioButton;
		[Glade.Widget] Gtk.RadioButton MgettyRadioButton;
		[Glade.Widget] Gtk.RadioButton EfaxRadioButton;
		[Glade.Widget] Gtk.Entry ModemEntry;
		[Glade.Widget] Gtk.Entry FaxNumberEntry;
		[Glade.Widget] Gtk.Entry DialPrefixEntry;
		[Glade.Widget] Gtk.Entry EmailAddressEntry;
		[Glade.Widget] Gtk.Entry HylafaxHostnameEntry;
		[Glade.Widget] Gtk.Entry HylafaxPortEntry;
		[Glade.Widget] Gtk.Entry HylafaxUsernameEntry;
		[Glade.Widget] Gtk.Notebook AgentTab;
		
		
		public GfaxFirstPrefs ()
		{
			Application.Init ();
			gxml = new Glade.XML (null, "gfax.glade","FirstTimeWindow",null);
			gxml.Autoconnect (this);
			FirstTimeDruid.ShowAll();
			Application.Run ();
		}

		private void on_HylafaxRadioButton_clicked (object o, EventArgs args) 
		{
			if (HylafaxRadioButton.Active)
				AgentTab.CurrentPage = 0;
			else 
				AgentTab.CurrentPage = 1;
		}

		private void on_FirstTimeWindow_delete_event (object o, DeleteEventArgs args) 
		{
			Application.Quit();
		}
		
		private void on_FirstTimeDruid_cancel (object o, EventArgs args) 
		{
			Application.Quit();
		}
		
		private void on_DruidPageFinish_finish (object o, EventArgs args) 
		{
		
			if (HylafaxRadioButton.Active) {
				Settings.TransmitAgent = "hylafax";
				Settings.Hostname = HylafaxHostnameEntry.Text;
				Settings.Port = HylafaxPortEntry.Text;
				Settings.Username = HylafaxUsernameEntry.Text;
			} else if (MgettyRadioButton.Active) {
				Settings.TransmitAgent = "mgetty";
				Settings.ModemPort = ModemEntry.Text;
			} else if (EfaxRadioButton.Active) {
				Settings.TransmitAgent = "efax";
				Settings.ModemPort = ModemEntry.Text;
			}
				
			Settings.FaxNumber = FaxNumberEntry.Text;
			Settings.PhonePrefix = DialPrefixEntry.Text;
			Settings.EmailAddress = EmailAddressEntry.Text;
			Settings.RunSetupAtStart = false;
			FirstTimeWindow.Hide();
			Application.Quit();
		}
	}

}
